// Left motor
#define MA_C        24  // PC3 - back
#define MA_D        25  // PC4 - forward
#define MA_ENABLE   9   // PD6  

// Right motor
#define MB_C        22  // PC0 - back
#define MB_D        23  // PC1 - forward
#define MB_ENABLE   8   // PD5

// LEDs and Sensors
#define LED_PIN    31
#define DIST_PIN   A1    

// Constants
#define MOTOR_DELAY    50   // miliseconds
#define MOTOR_MIN_PWM  120    // vaoarea intre 1 si 255 pentru enable la motorul care merge 100%
#define MOTOR_MAX_PWM  200   // caloarea intre MOTOR_MIN_PWM si 255 pentru motorul care merge 100%
#define MOTOR_DIFF     1  // cu cat la suta trebuie sa se miste motorul drept mai incet

#define DIST_TRESHOLD  170  // ADC value
#define BACK_LED       31

int motor_pwm;
boolean moving;

void motor_setup() 
{
    pinMode(MA_ENABLE, OUTPUT);
    pinMode(MA_C, OUTPUT);
    pinMode(MA_D, OUTPUT);
  
    pinMode(MB_ENABLE, OUTPUT);
    pinMode(MB_C, OUTPUT);
    pinMode(MB_D, OUTPUT);  
}

void motor_set_pwm(int pwm)
{
    if (pwm < MOTOR_MIN_PWM)
      pwm = MOTOR_MIN_PWM;
  
    if (pwm > MOTOR_MAX_PWM)
      pwm = MOTOR_MAX_PWM;
 
    Serial.println(pwm);
  
    motor_pwm = pwm;
    analogWrite(MA_ENABLE, pwm * MOTOR_DIFF);
    analogWrite(MB_ENABLE, pwm);
}

void motor_disable()
{
    moving = false;
    motor_set_pwm(0);
}

void motor_stop () 
{
    moving = false;
    digitalWrite(BACK_LED, LOW);
    digitalWrite(MA_C, LOW);
    digitalWrite(MA_D, LOW);
    digitalWrite(MB_C, LOW);
    digitalWrite(MB_D, LOW);
}

void motor_forward () 
{
    motor_set_pwm(MOTOR_MIN_PWM);
    motor_stop();
    
    delay(MOTOR_DELAY);
    moving = true;
    
    digitalWrite(MA_C, LOW);
    digitalWrite(MA_D, HIGH);
    digitalWrite(MB_C, LOW);
    digitalWrite(MB_D, HIGH);
}

void motor_back () 
{
    motor_set_pwm(MOTOR_MIN_PWM);
    motor_stop();
    
    delay(MOTOR_DELAY);
    moving = true;
    
    digitalWrite(BACK_LED, HIGH);
    
    digitalWrite(MA_C, HIGH);
    digitalWrite(MA_D, LOW);
    digitalWrite(MB_C, HIGH);
    digitalWrite(MB_D, LOW);
}

void motor_left () 
{
    motor_set_pwm(MOTOR_MIN_PWM);
    motor_stop();
   
    delay(MOTOR_DELAY);
    moving = true;
   
    digitalWrite(MA_C, LOW);
    digitalWrite(MA_D, HIGH);
    digitalWrite(MB_C, HIGH);
    digitalWrite(MB_D, LOW); 
}

void motor_right () 
{
    motor_set_pwm(MOTOR_MIN_PWM);
    motor_stop ();
   
    delay(MOTOR_DELAY);
    moving = true;
   
    digitalWrite(MA_C, HIGH);
    digitalWrite(MA_D, LOW);
    digitalWrite(MB_C, LOW);
    digitalWrite(MB_D, HIGH); 
}

void distance_init()
{
    pinMode(DIST_PIN, INPUT);  
}

int distance_get () 
{
    int val = analogRead(DIST_PIN);
    //delay(20);
    return val;
}

void setup()
{
    Serial.begin(9600);
    distance_init();
    motor_setup();
    pinMode(BACK_LED, OUTPUT);
}

void loop()
{
    int dist;
    
    Serial.println("Forward");
    motor_forward();
    
    // Cat timp nu are nimic in fata face busy waiting
    while ((dist = distance_get()) < DIST_TRESHOLD) {
        
        // Daca ne apropiem de un obstacol incetinim
        if (dist > DIST_TRESHOLD * 0.3) {
          
            motor_set_pwm(motor_pwm - 10);
          
        } else if (motor_pwm < MOTOR_MAX_PWM) {
          
            // Daca viteza este mai mica decat maximul setat creste
            motor_set_pwm(motor_pwm + 1);
        } 
        
        delay(20);
    }
  
    Serial.println("Back");
    motor_back();
    delay(350);
  
    Serial.println("Turn");
    
    if (rand() % 2) {
        motor_right();
    } else {
        motor_left();
    }
    
    delay(100);
    
    while (distance_get() > DIST_TRESHOLD);
}
